﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/// <summary>
/// This class stores the mob spawn information
/// </summary>
public class MobSpawnInfo
{
    /// <summary>
    /// Static variable will be set on game start with the number of different mobs
    /// </summary>
    private static int                  totalNumberOfMobTypes       { get; set; }

    /// <summary>
    /// The angle direction where the mobs cluster will spawn
    /// </summary>
    public float                        SpawnAngle                  { get; set; }
    /// <summary>
    /// Type of the mob cluster
    /// </summary>
    public Enumerators.Mob_Types        MobType                     { get; set; }
    /// <summary>
    /// Number of mobs on the cluster
    /// </summary>
    public int                          NumberOfMobs                { get; set; }
    
    static MobSpawnInfo()
    {
        //sets the current number of mob types
        MobSpawnInfo.totalNumberOfMobTypes = System.Enum.GetNames(typeof(Enumerators.Mob_Types)).Length;
    }

    public MobSpawnInfo()
    {
        this.SpawnAngle                 = Random.Range(0, 360);
        this.MobType                    = (Enumerators.Mob_Types)Random.Range(0, MobSpawnInfo.totalNumberOfMobTypes);
        this.NumberOfMobs               = Random.Range(GameManager.Singleton.CFG_MinNumberOfMobsPerSpawn, GameManager.Singleton.CFG_MaxNumberOfMobsPerSpawn);
    }
    public MobSpawnInfo(float spawnAngle, Enumerators.Mob_Types mobType, int numberOfMobs)
    {
        //check if the angle is < 0 or >= 360 and corrects the value
        if (spawnAngle < 0)
            this.SpawnAngle             = spawnAngle + 360;
        else if (spawnAngle >= 360)
            this.SpawnAngle             = spawnAngle - 360;
        else
            this.SpawnAngle             = spawnAngle;
        //check if the angle is < 0 or >= 360 and corrects the value

        this.MobType                    = mobType;
        this.NumberOfMobs               = numberOfMobs;
    }

    /// <summary>
    /// This method will return a new MobSpawnInfo.
    /// If GameManager.Singleton.CFG_MirrorMobSpawns is true the SpawnAngle will be mirrored acording to the screen.
    /// </summary>
    /// <returns></returns>
    public MobSpawnInfo Clone(Enumerators.Screen_Types screenType)
    {
        if (GameManager.Singleton.CFG_MirrorMobSpawns)
        {
            switch (screenType)
            {
                case Enumerators.Screen_Types.TopRight:
                    return new MobSpawnInfo(this.SpawnAngle, this.MobType, this.NumberOfMobs);
                case Enumerators.Screen_Types.TopLeft:
                    return new MobSpawnInfo(360 - this.SpawnAngle, this.MobType, this.NumberOfMobs);
                case Enumerators.Screen_Types.BottomRight:
                    return new MobSpawnInfo(180 - this.SpawnAngle, this.MobType, this.NumberOfMobs);
                case Enumerators.Screen_Types.BottomLeft:
                    return new MobSpawnInfo(180 + this.SpawnAngle, this.MobType, this.NumberOfMobs);
                default:
                    return new MobSpawnInfo(this.SpawnAngle, this.MobType, this.NumberOfMobs);
            }
        }
        else
            return new MobSpawnInfo(this.SpawnAngle, this.MobType, this.NumberOfMobs);
    }
}