﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Player : MonoBehaviour {

    public float playerspeed { get; set; }

    public Transform nose;
    public GameObject projectile;
    
    public float shootForce = 0f;
    private float shockForce = 5f;
    private bool shootBlocked;
    private bool swapBlocked;
    private bool shockBlocked;

    public bool IsPlayerDead { get; set; }

    private FoW.FogOfWarUnit fowUnit { get; set; }    

    public float VisionRadius
    {
        get
        {
            return fowUnit.radius;
        }
        set
        {
            fowUnit.radius = value;
        }
    }


    // Use this for initialization
    void Start()
    {
        playerspeed     = GameManager.Singleton.CFG_Default_PlayerSpeed;
        fowUnit         = gameObject.GetComponent<FoW.FogOfWarUnit>();      
        VisionRadius    = GameManager.Singleton.CFG_Default_Screen_VisionRadius;  
    }

    // Update is called once per frame
    void Update()
    {
        if (IsPlayerDead)
            return;

        VisionRadius = GameManager.Singleton.Current_ScreenConfiguration[(int)GameManager.Singleton.GetPlayerScreen(gameObject)].VisionRadius;
    }

    public void Shoot()
    {
        if (shootBlocked || IsPlayerDead)
            return;

        shootBlocked = true;
        GameObject Projectile = (GameObject)Instantiate(projectile, nose.position, nose.rotation);
        Projectile.GetComponent<Rigidbody>().AddForce(nose.forward * shootForce);
        Invoke("UnBlock", GameManager.Singleton.CFG_Player_Shoot_Cooldown);
            
    }

    void UnBlock()
    {
        shootBlocked = false;      
    }

    void UnSwap()
    {
        swapBlocked = false;
    }

    void UnShock()
    {
        shockBlocked = false;
    }

    void OnTriggerEnter(Collider other)
    {       
        if (other.gameObject.tag == "Enemy")
        {
            IsPlayerDead = true;
            GameManager.Singleton.PlayerDeath(gameObject);

            GetComponent<Rigidbody>().velocity = Vector3.zero;
        }
    }

    public void Swap()
    {
        if (swapBlocked || IsPlayerDead)
            return;

        swapBlocked = true;
        GameManager.Singleton.SwapPlayers(gameObject);
        Invoke("UnSwap", GameManager.Singleton.CFG_Player_SwapScreen_Cooldown);
    }

    public void Shockwave()
    {
        if (shockBlocked || IsPlayerDead)
            return;

        shockBlocked = true;
        Collider[] hitColliders = Physics.OverlapSphere(gameObject.transform.position, GameManager.Singleton.CFG_Player_ShockWave_Radius);

        for (int i = 0; i <= hitColliders.GetUpperBound(0); i++)
        {
            if (hitColliders[i].tag == "Enemy")
            {
                hitColliders[i].transform.parent = null;
                GameObject.Destroy(hitColliders[i].transform.gameObject);
            }
        }

        //int i = 0;
        //while (i < hitColliders.Length)
        //{
        //    hitColliders[i].GetComponent<Rigidbody>().AddForce(transform.position * shockForce);
        //    i++;
        //}
        Invoke("UnShock", GameManager.Singleton.CFG_Player_ShockWave_Cooldown);
    }

    
    


}
