﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class GameManager : MonoBehaviour
{
    /// <summary>
    /// Singleton instance of the GameManager Class so that it's available for other classes
    /// </summary>
    public static GameManager       Singleton                               { get; private set; }

    /// <summary>
    /// Represents the Player zero GameObject. Must be set on the Sceme
    /// </summary>
    public GameObject               CFG_Player_0                            = null;
    /// <summary>
    /// Represents the Player one GameObject. Must be set on the Sceme
    /// </summary>
    public GameObject               CFG_Player_1                            = null;
    /// <summary>
    /// Represents the Player two GameObject. Must be set on the Sceme
    /// </summary>
    public GameObject               CFG_Player_2                            = null;
    /// <summary>
    /// Represents the Player three GameObject. Must be set on the Sceme
    /// </summary>
    public GameObject               CFG_Player_3                            = null;

    /// <summary>
    /// Represents the center of the mob spawn circle for game area 0
    /// </summary>
    public GameObject               CFG_GameArea_0_SpawnOrigin              = null;
    /// <summary>
    /// Represents the center of the mob spawn circle for game area 1
    /// </summary>
    public GameObject               CFG_GameArea_1_SpawnOrigin              = null;
    /// <summary>
    /// Represents the center of the mob spawn circle for game area 2
    /// </summary>
    public GameObject               CFG_GameArea_2_SpawnOrigin              = null;
    /// <summary>
    /// Represents the center of the mob spawn circle for game area 3
    /// </summary>
    public GameObject               CFG_GameArea_3_SpawnOrigin              = null;

    /// <summary>
    /// Camera of game area 0
    /// </summary>
    public Camera                   CFG_Camera_GameArea_0                   = null;
    /// <summary>
    /// Camera of game area 1
    /// </summary>
    public Camera                   CFG_Camera_GameArea_1                   = null;
    /// <summary>
    /// Camera of game area 2
    /// </summary>
    public Camera                   CFG_Camera_GameArea_2                   = null;
    /// <summary>
    /// Camera of game area 3
    /// </summary>
    public Camera                   CFG_Camera_GameArea_3                   = null;


    /// <summary>
    /// Represents the prefab of the enemy 0
    /// </summary>
    public GameObject               CFG_Enemy_0_Prefab                      = null;
    /// <summary>
    /// Represents the prefab of the enemy 1
    /// </summary>
    public GameObject               CFG_Enemy_1_Prefab                      = null;

    /// <summary>
    /// The min number of mobs that will spawn in e each spawn cluster
    /// </summary>
    public int                      CFG_MinNumberOfMobsPerSpawn             = 1;
    /// <summary>
    /// The max number of mobs that will spawn in e each spawn cluster
    /// </summary>
    public int                      CFG_MaxNumberOfMobsPerSpawn             = 10;
    /// <summary>
    /// if set to true the mob spawn clusters of the left screen will be vertically mirrored
    /// </summary>
    public bool                     CFG_MirrorMobSpawns                     = true;
    /// <summary>
    /// Players will begin the game with this Vision Angle
    /// </summary>
    public float                    CFG_Default_Screen_VisionRadius         = 10.0f;
    /// <summary>
    /// Mobs will have this speed at the begin the game
    /// </summary>
    public float                    CFG_Default_Screen_MobSpeed             = 1.0f;
    /// <summary>
    /// Mobs will spawn at this rate in the begin the game
    /// </summary>
    public float                    CFG_Default_Screen_SpawnRate            = 1.0f;
    /// <summary>
    /// The number of screens will set how many screen configurations will be defined
    /// </summary>
    public int                      CFG_Number_Of_Screens                   = 1;
    /// <summary>
    /// Sets the distance the mobs will spawn
    /// </summary>
    public float                    CFG_MobSpawnRadius                      = 125.0f;
    /// <summary>
    /// Sets the Initial Player Speed
    /// </summary>
    public float                    CFG_Default_PlayerSpeed                 = 10.0f;

    public float                    CFG_Player_Shoot_Cooldown               = 0.3f;
    public float                    CFG_Player_SwapScreen_Cooldown          = 1f;
    public float                    CFG_Player_ShockWave_Cooldown           = 1f;
    public float                    CFG_Player_ShockWave_Radius             = 10f;


    /// <summary>
    /// Stores all the game areas configurations
    /// </summary>
    public List<ScreenCFG>          Current_ScreenConfiguration             { get; set; } 

    // Use this for initialization
    void Start()
    {
        GameManager.Singleton                       = this;

        ResetGame();
    }

    /// <summary>
    /// Resets all game variables
    /// </summary>
    public void ResetGame()
    {
        if (Current_ScreenConfiguration != null)
            Current_ScreenConfiguration.Clear();

        Current_ScreenConfiguration = new List<ScreenCFG>();

        for (int i = 0; i < CFG_Number_Of_Screens; i++)
        {
            Current_ScreenConfiguration.Add(new ScreenCFG(CFG_Default_Screen_VisionRadius, CFG_Default_Screen_MobSpeed, CFG_Default_Screen_SpawnRate));
        }

        switch (CFG_Number_Of_Screens)
        {
            case 4:
                CFG_Camera_GameArea_0.rect = new Rect(new Vector2(0.0f, 0.5f), new Vector2(0.5f, 0.5f));
                CFG_Camera_GameArea_0.orthographicSize = 15;
                CFG_Camera_GameArea_1.rect = new Rect(new Vector2(0.5f, 0.5f), new Vector2(0.5f, 0.5f));
                CFG_Camera_GameArea_1.orthographicSize = 15;
                CFG_Camera_GameArea_2.rect = new Rect(new Vector2(0.0f, 0.0f), new Vector2(0.5f, 0.5f));
                CFG_Camera_GameArea_2.orthographicSize = 15;
                CFG_Camera_GameArea_3.rect = new Rect(new Vector2(0.5f, 0.0f), new Vector2(0.5f, 0.5f));
                CFG_Camera_GameArea_3.orthographicSize = 15;
                break;
            case 3:
                CFG_Camera_GameArea_0.rect = new Rect(new Vector2(0.0f, 0.5f), new Vector2(0.5f, 0.5f));
                CFG_Camera_GameArea_0.orthographicSize = 15;
                CFG_Camera_GameArea_1.rect = new Rect(new Vector2(0.5f, 0.5f), new Vector2(0.5f, 0.5f));
                CFG_Camera_GameArea_1.orthographicSize = 15;
                CFG_Camera_GameArea_2.rect = new Rect(new Vector2(0.0f, 0.0f), new Vector2(0.5f, 0.5f));
                CFG_Camera_GameArea_2.orthographicSize = 15;
                CFG_Camera_GameArea_3.enabled = false;

                CFG_Player_3.GetComponent<Player>().IsPlayerDead = true;
                break;
            default:
                CFG_Camera_GameArea_0.rect = new Rect(new Vector2(0.0f, 0.0f), new Vector2(0.5f, 1.0f));
                CFG_Camera_GameArea_0.orthographicSize = 30;
                CFG_Camera_GameArea_1.rect = new Rect(new Vector2(0.5f, 0.0f), new Vector2(0.5f, 1.0f));
                CFG_Camera_GameArea_1.orthographicSize = 30;
                CFG_Camera_GameArea_2.enabled = false;
                CFG_Camera_GameArea_3.enabled = false;

                CFG_Player_2.GetComponent<Player>().IsPlayerDead = true;
                CFG_Player_3.GetComponent<Player>().IsPlayerDead = true;
                break;
        }
    }

    public void PlayerDeath(GameObject player)
    {
        PlayerDeath(GetPlayerScreen(player));
    }
    /// <summary>
    /// Set Game area as Dead. No Players can use the area until next game.
    /// </summary>
    /// <param name="screenType"></param>
    public void PlayerDeath(Enumerators.Screen_Types screenType)
    {
        Current_ScreenConfiguration[(int)screenType].ScreenDead = true;
        cleanSpawnQueue(screenType);
        cleanGameArea(screenType);
    }

    public void MobDeath(Enumerators.Screen_Types screenType)
    {
        Current_ScreenConfiguration[(int)screenType].MobKillCount++;
    }

    // Update is called once per frame
    void Update()
    {
        fillSpawnQueues();
        SpawnMobs();
    }
    
    /// <summary>
    /// Ensures that the Spawn Qeueues always have more then 20 MobSpawnInfos
    /// Both SpawnQueues are identical
    /// </summary>
    private void fillSpawnQueues()
    {
        if (checkSpawnQueuesBellowLimit(20))
        {
            lock (Current_ScreenConfiguration)
            {
                MobSpawnInfo mobSpawnInfo = null;

                while (checkSpawnQueuesBellowLimit(50))
                {
                    mobSpawnInfo = new MobSpawnInfo();

                    for (int i = 0; i < CFG_Number_Of_Screens; i++)
                    {
                        if(!Current_ScreenConfiguration[i].ScreenDead)
                            Current_ScreenConfiguration[i].SpawnQueue.Enqueue(mobSpawnInfo.Clone((Enumerators.Screen_Types)i));
                    }
                }
            }
        }
    }
    /// <summary>
    /// check if any of the spawn queues is bellow the min size
    /// </summary>
    /// <param name="minSize">check this min size</param>
    /// <returns></returns>
    private bool checkSpawnQueuesBellowLimit(int minSize)
    {
        for (int i = 0; i < CFG_Number_Of_Screens; i++)
        {
            if (!Current_ScreenConfiguration[i].ScreenDead && Current_ScreenConfiguration[i].SpawnQueue.Count < minSize)
                return true;
        }

        return false;
    }

    private void cleanSpawnQueue(Enumerators.Screen_Types screenType)
    {
        Current_ScreenConfiguration[(int)screenType].SpawnQueue.Clear();
    }
    private void cleanGameArea(Enumerators.Screen_Types screenType)
    {
        GameObject spawnOrigin = GetSpawnOrigin(screenType);

        if (spawnOrigin != null)
        {
            for (int i = 0; i < spawnOrigin.transform.childCount; i++)
            {
                GameObject.Destroy(spawnOrigin.transform.GetChild(i).gameObject);
            }
            spawnOrigin.transform.DetachChildren();
        }
    }

    /// <summary>
    /// returns the player GameObject based on the ScreenType.
    /// Used in the Mobs class so that mobs can find the correct targed
    /// </summary>
    /// <param name="screenType"></param>
    /// <returns></returns>
    public GameObject GetPlayer(Enumerators.Screen_Types screenType)
    {
        switch (screenType)
        {
            case Enumerators.Screen_Types.TopLeft:      return CFG_Player_0;
            case Enumerators.Screen_Types.TopRight:     return CFG_Player_1;
            case Enumerators.Screen_Types.BottomLeft:   return CFG_Player_2;
            case Enumerators.Screen_Types.BottomRight:  return CFG_Player_3;
            default:                                    return CFG_Player_0;
        }
    }
    public GameObject GetSpawnOrigin(Enumerators.Screen_Types screenType)
    {
        switch (screenType)
        {
            case Enumerators.Screen_Types.TopLeft:      return CFG_GameArea_0_SpawnOrigin;
            case Enumerators.Screen_Types.TopRight:     return CFG_GameArea_1_SpawnOrigin;
            case Enumerators.Screen_Types.BottomLeft:   return CFG_GameArea_2_SpawnOrigin;
            case Enumerators.Screen_Types.BottomRight:  return CFG_GameArea_3_SpawnOrigin;
            default:                                    return CFG_GameArea_0_SpawnOrigin;
        }
    }
    public GameObject GetMobPrefab(Enumerators.Mob_Types mobType)
    {
        switch (mobType)
        {
            case Enumerators.Mob_Types.Cube:            return CFG_Enemy_0_Prefab;
            case Enumerators.Mob_Types.Sphere:          return CFG_Enemy_1_Prefab;
            default:                                    return CFG_Enemy_0_Prefab;
        }
    }
    public Enumerators.Screen_Types GetPlayerScreen(GameObject player)
    {
        if (CFG_Player_0 == player)             return Enumerators.Screen_Types.TopLeft;
        else if (CFG_Player_1 == player)        return Enumerators.Screen_Types.TopRight;
        else if (CFG_Player_2 == player)        return Enumerators.Screen_Types.BottomLeft;
        else if (CFG_Player_3 == player)        return Enumerators.Screen_Types.BottomRight;
        else                                    return Enumerators.Screen_Types.TopLeft;
    }
    public int GetNumberActiveGameAreas()
    {
        int numAcriveScreens = 0;

        for (int i = 0; i < CFG_Number_Of_Screens; i++)
        {
            if (!Current_ScreenConfiguration[i].ScreenDead)
                numAcriveScreens++;
        }

        return numAcriveScreens;
    }

    /// <summary>
    /// checks all games areas and spawn mobs at the right time
    /// </summary>
    public void SpawnMobs()
    {
        try
        {
            for (int i = 0; i < CFG_Number_Of_Screens; i++)
            {
                if (!Current_ScreenConfiguration[i].ScreenDead && Current_ScreenConfiguration[i].NextSpawnTime < Time.time)
                {
                    Current_ScreenConfiguration[i].LastSpawnTime        = Time.time;

                    MobSpawnInfo    mobSpawnInfo                        = Current_ScreenConfiguration[i].SpawnQueue.Dequeue();
                    GameObject      spawnOrigin                         = GetSpawnOrigin((Enumerators.Screen_Types)i);      
                    Vector3         mobSpawnPossition                   = (new Vector3(Mathf.Sin(mobSpawnInfo.SpawnAngle * Mathf.Deg2Rad), 0, Mathf.Cos(mobSpawnInfo.SpawnAngle * Mathf.Deg2Rad)) * CFG_MobSpawnRadius) + spawnOrigin.transform.position;
                    GameObject      mob                                 = (GameObject)Instantiate(GetMobPrefab(mobSpawnInfo.MobType), mobSpawnPossition, Quaternion.identity);
                    mob.transform.parent                                = spawnOrigin.transform;

                    mob.GetComponent<Mob>().SetMobConfig(mobSpawnInfo.MobType, (Enumerators.Screen_Types)i);
                    
                }
            }
        }
        catch (System.Exception ex)
        {
            Debug.Log(ex.Message);
        }
        
    }

    
    public void SwapPlayers(GameObject player)
    {
        if (GetNumberActiveGameAreas() > 1)
        {
            Enumerators.Screen_Types playerScreen = GetPlayerScreen(player);
            Enumerators.Screen_Types swapScreen = playerScreen;

            while(playerScreen == swapScreen)
            {
                swapScreen = (Enumerators.Screen_Types)Random.Range(0, CFG_Number_Of_Screens);
                if (Current_ScreenConfiguration[(int)swapScreen].ScreenDead)
                    swapScreen = playerScreen;
            }

            GameObject swapPlayer                                           = GetPlayer(swapScreen);
            UnityStandardAssets.Utility.FollowTarget playerCameraFollow     = null;
            UnityStandardAssets.Utility.FollowTarget swapCameraFollow       = null;

            switch (playerScreen)
            {
                case Enumerators.Screen_Types.TopLeft:      CFG_Player_0 = swapPlayer; playerCameraFollow = CFG_Camera_GameArea_0.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
                case Enumerators.Screen_Types.TopRight:     CFG_Player_1 = swapPlayer; playerCameraFollow = CFG_Camera_GameArea_1.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
                case Enumerators.Screen_Types.BottomLeft:   CFG_Player_2 = swapPlayer; playerCameraFollow = CFG_Camera_GameArea_2.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
                case Enumerators.Screen_Types.BottomRight:  CFG_Player_3 = swapPlayer; playerCameraFollow = CFG_Camera_GameArea_3.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
            }

            switch (swapScreen)
            {
                case Enumerators.Screen_Types.TopLeft:      CFG_Player_0 = player; swapCameraFollow = CFG_Camera_GameArea_0.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
                case Enumerators.Screen_Types.TopRight:     CFG_Player_1 = player; swapCameraFollow = CFG_Camera_GameArea_1.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
                case Enumerators.Screen_Types.BottomLeft:   CFG_Player_2 = player; swapCameraFollow = CFG_Camera_GameArea_2.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
                case Enumerators.Screen_Types.BottomRight:  CFG_Player_3 = player; swapCameraFollow = CFG_Camera_GameArea_3.GetComponent<UnityStandardAssets.Utility.FollowTarget>(); break;
            }
            
            Vector3 playerPosition          = player.transform.position;
            player.transform.position       = swapPlayer.transform.position;
            swapPlayer.transform.position   = playerPosition;

            playerCameraFollow.target = swapPlayer.transform;
            swapCameraFollow.target = player.transform;
        }
    }
}